/*


*/
/*
   prompting shell version 6
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/wait.h>

char * next_file(char *);
char ** next_cmd(char *);
char * makestring(char *);
void execute(char *, char *[], char *[], char *);

#define  MAXARGS   20   /* cmdline args */
#define  ARGLEN   100   /* token length */

int main()
{
   char *source = NULL, *dest = NULL;
   char **arglist1 = NULL, **arglist2 = NULL;

   while (1)
   {
      if ( (source = next_file("Source file")) != NULL )
         if ( (arglist1 = next_cmd("Cmd 1")) != NULL )
            if ( (arglist2 = next_cmd("Cmd 2")) != NULL )
               if ( (dest = next_file("Dest file")) != NULL )
               {
                  execute(source, arglist1, arglist2, dest);
                  /* free the argument lists */
                  char **cpp = arglist1;
                  while ( *cpp )
                     free( *cpp++ ); /* free cmd1's args */
                  cpp = arglist2;
                  while ( *cpp )
                     free( *cpp++ ); /* free cmd2's args */
                  free( arglist1 );
                  free( arglist2 );
                  free( source );
                  free( dest );
               }
               else
                  break;
            else
               break;
         else
            break;
      else
         break;
   }
   return 0;
}


/*
 * purpose: read next file name
 * returns: dynamically allocated string holding the file name
 *        : NULL at EOF or ENTER
 */
char * next_file(char *prompt)
{
   char *filename = NULL;
   char namebuf[ARGLEN]; /* read stuff here */

   /* user can type either a file name, ENTER, or CTRL-D */
   printf("%s? ", prompt);
   if ( fgets(namebuf, ARGLEN, stdin) && *namebuf != '\n' )
   {
      /* user typed a file name */
      filename = makestring(namebuf);
   }
   else /* user typed ENTER or CTRL-D (EOF) */
   {
      filename = NULL;
   }
   return filename;
}


/*
 * purpose: read next command line
 * returns: dynamically allocated array holding command line arguments
 *        : NULL at EOF
 */
char ** next_cmd(char *prompt)
{
   char **arglist;      /* an array of ptrs */
   int  numargs;        /* index into array */
   char argbuf[ARGLEN]; /* read stuff here */

   /* get arglist[0] */
   /* user can type either a command name, ENTER, or CTRL-D */
   numargs = 0;
   printf("%s Arg[%d]? ", prompt, numargs);
   if ( fgets(argbuf, ARGLEN, stdin) && *argbuf != '\n' )
   {
      /* user typed a command name */
      arglist = malloc( sizeof(char*) * (MAXARGS+1) );
      arglist[numargs++] = makestring(argbuf);
   }
   else /* user typed ENTER or CTRL-D (EOF) */
   {
      return NULL;
   }

   /* get the rest of the arglist */
   while ( numargs < MAXARGS )
   {
      /* user can type either an argument, ENTER, or CTRL-D */
      printf("%s Arg[%d]? ", prompt, numargs);
      if ( fgets(argbuf, ARGLEN, stdin) && *argbuf != '\n' )
      {
         /* user typed an argument */
         arglist[numargs++] = makestring(argbuf);
      }
      else /* user typed ENTER or CTRL-D (EOF) */
      {
         arglist[numargs] = NULL;   /* close list */
         return arglist;
      }
   }
   arglist[MAXARGS] = NULL;   /* close list */
   return arglist;
}


/*
 * trim off newline and create storage for the string
 */
char * makestring( char *buf )
{
   char  *cp;

   buf[strlen(buf)-1] = '\0';    /* trim newline */
   cp = malloc( strlen(buf)+1 ); /* get memory   */
   if ( cp == NULL )             /* or die       */
   {
      fprintf(stderr,"no memory\n");
      exit(1);
   }
   strcpy(cp, buf); /* copy chars */
   return cp;       /* return ptr */
}


/*
   Set up and execute the pipeline.
*/
void execute(char *source, char *arglist1[], char *arglist2[], char *dest)
{




}
